@@ -28,6 +28,8 @@
 #include <glade/glade-widget-tree.h>
 
 struct _GladeXMLPrivate {
+	GladeWidgetTree *tree; /* the tree for this GladeXML */
+
 	GtkTooltips *tooltips; /* if not NULL, holds all tooltip info */
 
 	/*
@@ -60,6 +62,26 @@ struct _GladeXMLPrivate {
 	 */
 	GtkAccelGroup *accel_group;
 	GSList *uline_accels;
+
+	/* an accel intended for the parent of a widget */
+	guint parent_accel;
+	/* a list of label uline accels for widgets that don't exist yet */
+	GList *focus_ulines;
+};
+
+typedef struct _GladeFocusULine GladeFocusULine;
+struct _GladeFocusULine {
+	const gchar *widget_name;
+	guint key;
+};
+
+typedef struct _GladeSignalData GladeSignalData;
+struct _GladeSignalData {
+	GtkObject *signal_object;
+	char *signal_name;
+	char *signal_data;    /* this isn't actually used, but is in the XML */
+	char *connect_object; /* or NULL if there is none */
+	gboolean signal_after;
 };
 
 /*
@@ -76,9 +98,5 @@ const char *glade_get_widget_name      (GtkWidget *widget);
 /* get the name of the widget (dot separated heirachy) */
 const char *glade_get_widget_long_name (GtkWidget *widget);
 
-/* from glade-keys.c */
-/* perform string->int keysym conversion */
-guint glade_key_get(char *str);
-
 #endif
 
