@@ -26,6 +26,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
 #include <gtk/gtk.h>
 
@@ -39,7 +41,7 @@
  * negative value on error.
  */
 static int my_xmlSAXParseFile(xmlSAXHandlerPtr sax, void *user_data,
-		      const char *filename) {
+			      const char *filename) {
     int ret = 0;
     xmlParserCtxtPtr ctxt;
     
@@ -61,9 +63,33 @@ static int my_xmlSAXParseFile(xmlSAXHandlerPtr sax, void *user_data,
     return ret;
 }
 
+static int my_xmlSAXParseMemory(xmlSAXHandlerPtr sax, void *user_data,
+				char *buffer, int size) {
+    int ret = 0;
+    xmlParserCtxtPtr ctxt;
+    
+    ctxt = xmlCreateMemoryParserCtxt(buffer, size);
+    if (ctxt == NULL) return -1;
+    ctxt->sax = sax;
+    ctxt->userData = user_data;
+    
+    xmlParseDocument(ctxt);
+    
+    if (ctxt->wellFormed)
+	ret = 0;
+    else
+	ret = -1;
+    if (sax != NULL)
+	ctxt->sax = NULL;
+    xmlFreeParserCtxt(ctxt);
+    
+    return ret;
+}
+
 static GladeWidgetTree *glade_widget_tree_new(void) {
     GladeWidgetTree *self = g_new0(GladeWidgetTree, 1);
 
+    self->ref = 1;
     self->names = g_hash_table_new(g_str_hash, g_str_equal);
 
     return self;
@@ -132,14 +158,29 @@ static void glade_widget_info_free(GladeWidgetInfo *info) {
 }
 
 /**
- * glade_widget_tree_free
+ * glade_widget_tree_ref
  * @tree: the GladeWidgetTree structure
  *
- * Free a GladeWidgetTree structure.
+ * Increment the reference count of a GladeWidgetTree structure.
+ * Returns: the tree argument.
  */
-void glade_widget_tree_free(GladeWidgetTree *tree) {
+GladeWidgetTree *glade_widget_tree_ref(GladeWidgetTree *tree) {
+    tree->ref++;
+    return tree;
+}
+
+/**
+ * glade_widget_tree_unref
+ * @tree: the GladeWidgetTree structure
+ *
+ * Decrement the reference count of a GladeWidgetTree structure.
+ */
+void glade_widget_tree_unref(GladeWidgetTree *tree) {
     GList *tmp;
 
+    if (--(tree->ref) > 0)
+	return;
+
     /* free the styles */
     for (tmp = tree->styles; tmp; tmp = tmp->next) {
 	GladeStyleInfo *inf = tmp->data;
@@ -671,12 +712,40 @@ static xmlSAXHandler gladeSAXParser = {
  */
 GladeWidgetTree *glade_widget_tree_parse_file(const char *file) {
     GladeParseState state;
+    struct stat statbuf;
 
     state.tree = NULL;
     if (my_xmlSAXParseFile(&gladeSAXParser, &state, file) < 0) {
 	g_warning("document not well formed!");
 	if (state.tree)
-	    glade_widget_tree_free(state.tree);
+	    glade_widget_tree_unref(state.tree);
+	return NULL;
+    }
+    /* set the modification time of the file ... */
+    if (stat(file, &statbuf) >= 0)
+	state.tree->mtime = statbuf.st_mtime;
+    return state.tree;
+}
+
+/**
+ * glade_widget_tree_parse_memory
+ * @buffer: the in memory buffer holding the XML document
+ * @size: the size of the buffer
+ *
+ * This routine will parse an in memory buffer containing Glade XML, and
+ * produce a GladeWidgetTree structure containing the information in this
+ * buffer.
+ *
+ * Returns: the GladeWidgetTree structure, or NULL on error.
+ */
+GladeWidgetTree *glade_widget_tree_parse_memory(char *buffer, int size) {
+    GladeParseState state;
+
+    state.tree = NULL;
+    if (my_xmlSAXParseMemory(&gladeSAXParser, &state, buffer, size) < 0) {
+	g_warning("document not well formed!");
+	if (state.tree)
+	    glade_widget_tree_unref(state.tree);
 	return NULL;
     }
     return state.tree;
