@@ -102,6 +102,8 @@ glade_xml_init (GladeXML *self)
 	priv->toplevel = NULL;
 	priv->accel_group = NULL;
 	priv->uline_accels = NULL;
+	priv->parent_accel = 0;
+	priv->focus_ulines = NULL;
 }
 
 /**
@@ -154,7 +156,7 @@ glade_xml_new_with_domain(const char *fname, const char *root,
 	GladeXML *self = gtk_type_new(glade_xml_get_type());
 
 	if (!glade_xml_construct(self, fname, root, domain)) {
-		gtk_object_destroy(GTK_OBJECT(self));
+		gtk_object_unref(GTK_OBJECT(self));
 		return NULL;
 	}
 	return self;
@@ -181,6 +183,7 @@ glade_xml_construct (GladeXML *self, const char *fname, const char *root,
 	if (!tree)
 		return FALSE;
 
+	self->priv->tree = tree;
 	if (self->textdomain) g_free(self->textdomain);
 	self->textdomain = g_strdup(domain);
 	if (self->filename)
@@ -194,6 +197,42 @@ glade_xml_construct (GladeXML *self, const char *fname, const char *root,
 	return TRUE;
 }
 
+/**
+ * glade_xml_new_from_memory:
+ * @buffer: the memory buffer containing the XML document.
+ * @size: the size of the buffer.
+ * @root: the widget node in @buffer to start building from (or %NULL)
+ * @domain: the translation domain to use for this interface (or %NULL)
+ *
+ * Creates a new GladeXML object (and the corresponding widgets) from the
+ * buffer @buffer.  Optionally it will only build the interface from the
+ * widget node @root (if it is not %NULL).  This feature is useful if you
+ * only want to build say a toolbar or menu from the XML document, but not the
+ * window it is embedded in.
+ *
+ * Returns: the newly created GladeXML object, or NULL on failure.
+ */
+GladeXML *glade_xml_new_from_memory(char *buffer, int size, const char *root,
+				    const char *domain)
+{
+	GladeXML *self;
+	GladeWidgetTree *tree = glade_widget_tree_parse_memory(buffer, size);
+
+	if (!tree)
+		return NULL;
+	self = gtk_type_new(glade_xml_get_type());
+
+	self->priv->tree = tree;
+	self->textdomain = g_strdup(domain);
+	self->filename = NULL;
+	glade_xml_build_interface(self, tree, root);
+
+	if (self->priv->tooltips)
+		gtk_tooltips_enable(self->priv->tooltips);
+
+	return self;
+}
+
 /**
  * glade_xml_signal_connect:
  * @self: the GladeXML object
@@ -524,6 +563,7 @@ glade_xml_set_toplevel(GladeXML *xml, GtkWindow *window)
 	xml->priv->toplevel = window;
 	/* new toplevel needs new accel group */
 	xml->priv->accel_group = NULL;
+	xml->priv->parent_accel = 0;
 }
 
 /**
@@ -590,9 +630,65 @@ glade_xml_get_uline_accel(GladeXML *xml) {
 		return NULL;
 }
 
+/**
+ * glade_xml_handle_label_accel:
+ * @xml: the GladeXML object.
+ * @target: the target widget name (or %NULL).
+ * @key: the key code for the accelerator.
+ *
+ * This function is called by the GtkLabel creation routine to register an
+ * underline accelerator for the label.  If the target widget exists, the
+ * accelerator is connected to the grab_focus signal.  If it does not exist
+ * yet, the (target, key) pair is saved so that it can be attached when the
+ * widget is created.
+ * If @target is %NULL, then the accelerator should be attached to one of
+ * the parents of this widget (typically a GtkButton).
+ */
+void
+glade_xml_handle_label_accel(GladeXML *xml, const gchar *target, guint key)
+{
+	if (target) {
+		GtkWidget *twidget = glade_xml_get_widget(xml, target);
+
+		if (twidget)
+			gtk_widget_add_accelerator(twidget, "grab_focus",
+						   glade_xml_ensure_accel(xml),
+						   key, GDK_MOD1_MASK, 0);
+		else {
+			GladeFocusULine *uline = g_new(GladeFocusULine, 1);
+
+			uline->widget_name = target;
+			uline->key = key;
+			xml->priv->focus_ulines =
+				g_list_prepend(xml->priv->focus_ulines, uline);
+		}
+	} else
+		xml->priv->parent_accel = key;
+}
+
+/**
+ * glade_xml_get_parent_accel:
+ * @xml: the GladeXML object.
+ *
+ * This function gets the latest label underline accelerator directed at the
+ * parent widget.  If there is no accelerator waiting, 0 is returned.  If
+ * this function is called twice in a row, the second call will always return
+ * 0.
+ *
+ * Returns: the key code for the accelerator, or zero.
+ */
+guint
+glade_xml_get_parent_accel(GladeXML *xml)
+{
+	guint key = xml->priv->parent_accel;
+
+	xml->priv->parent_accel = 0;
+	return key;
+}
+
 /**
  * glade_xml_gettext:
- * @xml: the GladeXML widget.
+ * @xml: the GladeXML object.
  * @msgid: the string to translate.
  *
  * This function is a wrapper for gettext, that uses the translation domain
@@ -691,6 +787,7 @@ glade_xml_destroy(GtkObject *object)
 	if (self->textdomain)
 		g_free(self->textdomain);
 
+	glade_widget_tree_unref(priv->tree);
 	/* strings are owned in the cached GladeWidgetTree structure */
 	g_hash_table_destroy(priv->name_hash);
 	/* strings belong to individual widgets -- don't free them */
@@ -890,6 +987,23 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
 	glade_xml_add_signals(self, widget, info);
 	glade_xml_add_accels(self, widget, info);
 
+	for (tmp = self->priv->focus_ulines; tmp; tmp = tmp->next) {
+		GladeFocusULine *uline = tmp->data;
+
+		if (!strcmp(uline->widget_name, info->name)) {
+			/* it is for us ... */
+			gtk_widget_add_accelerator(widget, "grab_focus",
+						glade_xml_ensure_accel(self),
+						uline->key, GDK_MOD1_MASK, 0);
+			tmp = tmp->next;
+			self->priv->focus_ulines =
+				g_list_remove(self->priv->focus_ulines, uline);
+			g_free(uline);
+		}
+		if (!tmp)
+			break;
+	}
+
 	gtk_widget_set_name(widget, info->name);
 	if (info->tooltip) {
 		if (self->priv->tooltips == NULL)
@@ -989,32 +1103,35 @@ glade_get_adjustment(GladeWidgetInfo *info)
 
 	for (tmp = info->attributes; tmp != NULL; tmp = tmp->next) {
 		GladeAttribute *attr = tmp->data;
+		gchar *name = attr->name;
 
-		if (attr->name[0] == 'h')
-			switch (attr->name[1]) {
-			case 'l':
-				if (!strcmp(attr->name, "hlower"))
-					hlower = g_strtod(attr->value, NULL);
-				break;
-			case 'p':
-				if (!strcmp(attr->name, "hpage"))
-					hpage = g_strtod(attr->value, NULL);
-				else if (!strcmp(attr->name, "hpage_size"))
-					hpage_size=g_strtod(attr->value, NULL);
-				break;
-			case 's':
-				if (!strcmp(attr->name, "hstep"))
-					hstep = g_strtod(attr->value, NULL);
-				break;
-			case 'u':
-				if (!strcmp(attr->name, "hupper"))
-					hupper = g_strtod(attr->value, NULL);
-				break;
-			case 'v':
-				if (!strcmp(attr->name, "hvalue"))
-					hvalue = g_strtod(attr->value, NULL);
-				break;
-			}
+		if (name[0] == 'h')
+			name++;
+
+		switch (name[0]) {
+		case 'l':
+			if (!strcmp(name, "lower"))
+				hlower = g_strtod(attr->value, NULL);
+			break;
+		case 'p':
+			if (!strcmp(name, "page"))
+				hpage = g_strtod(attr->value, NULL);
+			else if (!strcmp(name, "page_size"))
+				hpage_size=g_strtod(attr->value, NULL);
+			break;
+		case 's':
+			if (!strcmp(name, "step"))
+				hstep = g_strtod(attr->value, NULL);
+			break;
+		case 'u':
+			if (!strcmp(name, "upper"))
+				hupper = g_strtod(attr->value, NULL);
+			break;
+		case 'v':
+			if (!strcmp(name, "value"))
+				hvalue = g_strtod(attr->value, NULL);
+			break;
+		}
 	}
 	return GTK_ADJUSTMENT (gtk_adjustment_new(hvalue, hlower, hupper,
 						  hstep, hpage, hpage_size));
