@@ -36,7 +36,6 @@ extern "C" {
 
 typedef struct _GladeXML GladeXML;
 typedef struct _GladeXMLClass GladeXMLClass;
-typedef struct _GladeSignalData GladeSignalData;
 typedef struct _GladeXMLPrivate GladeXMLPrivate;
 
 struct _GladeXML {
@@ -54,18 +53,12 @@ struct _GladeXMLClass {
   GtkDataClass parent_class;
 };
 
-struct _GladeSignalData {
-  GtkObject *signal_object;
-  char *signal_name;
-  char *signal_data; /* this isn't actually used, but it is in the XML files */
-  char *connect_object; /* or NULL if there is none */
-  gboolean signal_after;
-};
-
 GtkType glade_xml_get_type    (void);
 GladeXML *glade_xml_new       (const char *fname, const char *root);
 GladeXML *glade_xml_new_with_domain (const char *fname, const char *root,
 				     const char *domain);
+GladeXML *glade_xml_new_from_memory (char *buffer, int size, const char *root,
+				     const char *domain);
 gboolean glade_xml_construct  (GladeXML *self, const char *fname,
 			       const char *root, const char *domain);
 
